package javax.swing.table;

import java.util.EventListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.EventListenerList;
import java.io.Serializable;

public abstract class AbstractTableModel implements TableModel, Serializable
{
    protected EventListenerList listenerList;
    
    public AbstractTableModel() {
        this.listenerList = new EventListenerList();
    }
    
    @Override
    public String getColumnName(int i) {
        String string = "";
        while (i >= 0) {
            string = (char)((char)(i % 26) + 65) + string;
            i = i / 26 - 1;
        }
        return string;
    }
    
    public int findColumn(String s) {
        int i = 0;
        while (i < this.getColumnCount()) {
            if (s.equals(this.getColumnName(i))) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    @Override
    public Class<?> getColumnClass(int n) {
        return Object.class;
    }
    
    @Override
    public boolean isCellEditable(int n, int n2) {
        return false;
    }
    
    @Override
    public void setValueAt(Object o, int n, int n2) {
    }
    
    @Override
    public void addTableModelListener(TableModelListener l) {
        this.listenerList.add(TableModelListener.class, l);
    }
    
    @Override
    public void removeTableModelListener(TableModelListener l) {
        this.listenerList.remove(TableModelListener.class, l);
    }
    
    public TableModelListener[] getTableModelListeners() {
        return (TableModelListener[])this.listenerList.getListeners(TableModelListener.class);
    }
    
    public void fireTableDataChanged() {
        this.fireTableChanged(new TableModelEvent(this));
    }
    
    public void fireTableStructureChanged() {
        this.fireTableChanged(new TableModelEvent(this, -1));
    }
    
    public void fireTableRowsInserted(int firstRow, int lastRow) {
        this.fireTableChanged(new TableModelEvent(this, firstRow, lastRow, -1, 1));
    }
    
    public void fireTableRowsUpdated(int firstRow, int lastRow) {
        this.fireTableChanged(new TableModelEvent(this, firstRow, lastRow, -1, 0));
    }
    
    public void fireTableRowsDeleted(int firstRow, int lastRow) {
        this.fireTableChanged(new TableModelEvent(this, firstRow, lastRow, -1, -1));
    }
    
    public void fireTableCellUpdated(int n, int column) {
        this.fireTableChanged(new TableModelEvent(this, n, n, column));
    }
    
    public void fireTableChanged(TableModelEvent tableModelEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TableModelListener.class) {
                ((TableModelListener)listenerList[i + 1]).tableChanged(tableModelEvent);
            }
        }
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> t) {
        return this.listenerList.getListeners(t);
    }
}
